'use strict';

const assert = require('chai').assert;
const hl = require('highland');
const db = require('tm-apps-db')(require('config').db);
const R = require('ramda');
let rewire = require("rewire");
let app = rewire("../../src/server");
let request = require("supertest");
let config = require("config");
let nock = require("nock");
let server = request(app(config, void 0, db));

const tag = {
    "id": "some:tag:identifier",
    "tag_id": "nationals.1",
    "standardized_name": "standard-name",
    "name": "Great Name",
    "article_count": 44
};

const createArticleTags = tuple => {
    const publishedDate = tuple[1];
    const id = tuple[0];

    const articles_tags = {
        "article_id": id,
        "tag_id": "some:tag:identifier",
        "published_date": publishedDate
    };

    return hl(db.article_tag.upsert(articles_tags));
};

const createBasicArticle = id => {
    return {
        "data": {
            "attributes": {
            },
            "id": Number(id)
        }
    };
};

function randomDate(id) {
    const now = new Date();
    const since = new Date(2016, 10, 10);
    return [id, new Date(since.getTime() + Math.random() * (now.getTime() - since.getTime()))];
}

before(() => {
    nock("http://apps-test-endpoint.com")
        .persist()
        .get(/\/(?:nationals|regionals)\/\d+\/articles\/(\d+)/)
        .reply(200, uri => createBasicArticle(R.match(/\/(?:nationals|regionals)\/\d+\/articles\/(\d+)/, uri)[1]));
});

after(() => {
    nock.restore();
});

describe("Tags Component", () => {

    let sortedIds;

    before(function (done) {
        this.timeout(6000);
        db.sequelize.sync({force: true}).then(function () {
            done();
        });
    });

    before(done => {
        const arrayOfIdsAndDates = R.range(200, 260).map(integer => `nationals.${integer}`).map(randomDate);

        sortedIds = R.take(40, R.sort((tuple1, tuple2) => {
            return tuple2[1] - tuple1[1];
        }, arrayOfIdsAndDates));

        const createArticlesStreams = arrayOfIdsAndDates.map(createArticleTags);
        let now = new Date();

        hl(createArticlesStreams)
            .merge()
            .collect()
            .flatMap(() => createArticleTags([
                "nationals.12345",
                now.setDate(now.getDate() + 14)
            ]))
            .flatMap(() => hl(db.tag.upsert(tag)))
            .toCallback(done);
    });

    it("Gets the 30 articles in section 71", done => {
        server
            .get("/nationals/2/tags/1")
            .end((err, res) => {
                // check that articles published in future are not included
                assert.notEqual(res.body.included[0].id, 12345);
                
                assert.equal(res.header['content-type'], 'application/json; charset=utf-8');
                assert.equal(res.body.data.type, "articles");
                assert.equal(res.body.included.length, 30);
                for (let i = 0; i < 30; i += 1) {
                    assert.equal(`nationals.${res.body.included[i].data.id}`, sortedIds[i][0]);
                }
                done();
            });
    });

    it("Handles 404s correctly", done => {
        server
            .get("/nationals/2/tags/6814768416841")
            .end((err, res) => {
                assert.equal(res.body.message, "nationals/2/tags/6814768416841 not available");
                assert.equal(res.body.code, "NotFoundError");
                done();
            });
    });
});